/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <string.h>
#include "h.h"

static char buf[BUFSIZE];


/*
** m_functions execute protocol messages on this server:
**
**	cptr	is always NON-NULL, pointing to a *LOCAL* client
**		structure (with an open socket connected!). This
**		identifies the physical socket where the message
**		originated (or which caused the m_function to be
**		executed--some m_functions may call others...).
**
**	sptr	is the source of the message, defined by the
**		prefix part of the message if present. If not
**		or prefix not found, then sptr==cptr.
**
**		(!IsServer(cptr)) => (cptr == sptr), because
**		prefixes are taken *only* from servers...
**
**		(IsServer(cptr))
**			(sptr == cptr) => the message didn't
**			have the prefix.
**
**			(sptr != cptr && IsServer(sptr) means
**			the prefix specified servername. (?)
**
**			(sptr != cptr && !IsServer(sptr) means
**			that message originated from a remote
**			user (not local).
**
**		combining
**
**		(!IsServer(sptr)) means that, sptr can safely
**		taken as defining the target structure of the
**		message in this server.
**	*Always* true (if 'parse' and others are working correct):
**
**	1)	sptr->from == cptr  (note: cptr->from == cptr)
**
**	2)	MyConnect(sptr) <=> sptr == cptr (e.g. sptr
**		*cannot* be a local connection, unless it's
**		actually cptr!). [MyConnect(x) should probably
**		be defined as (x == x->from) --msa ]
**
**	parc	number of variable parameter strings (if zero,
**		parv is allowed to be NULL)
**
**	parv	a NULL terminated list of parameter pointers,
**
**			parv[0], sender (prefix string), if not present
**				this points to an empty string.
**			parv[1]...parv[parc-1]
**				pointers to additional parameters
**			parv[parc] == NULL, *always*
**
**		note:	it is guaranteed that parv[0]..parv[parc-1] are all
**			non-NULL pointers.
*/

 /*
    ** m_help
    **  parv[0] = sender prefix
    ** --
    **  parv[1] = actual command
  */
int
m_help (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
  int i;

  sendto_one (sptr, ":%s 372 %s :\037HELP <command>\037   for quick scan.",
	      me.name, parv[0]);
  sendto_one (sptr,
	      ":%s 372 %s :\002<arg>\002 means compulsory argument, \002[<arg>]\002 means optional",
	      me.name, parv[0]);

  if (parc > 1)
    {
      if (!mycmp (parv[1], "WHO"))
	{
	  sendto_one (sptr,
		      ":%s 372 %s :WHO \"masklist\" [\"flaglist\"[,\"quertype\"]] [\"masklist\"]",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\"masklist\"   ::=   \"nowildlist\" | \"wildmask\"",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\"wildmask\"   ::= \"mask\" (matched against all the fields)",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\"nowildlist\" ::= ( channel | nick ) [ , ( channel | nick ) ]...",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\"flaglist\"   ::= [(filterflag)]... ['%%' (formatflag)... ]",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :\"filterflag\" ::= o | h | i | w",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\"formatflag\" ::= c | u | i | h | s | n | f | d | r | t");
	  sendto_one (sptr,
		      ":%s 372 %s :\"querytype\"  ::= Any number in the range 0-999 (defaults to 0)");

	  return 0;
	}
      else if (!mycmp (parv[1], "MODE") || !mycmp (parv[1], "M"))
	{
	  sendto_one (sptr,
		      ":%s 372 %s :\037Channel\037: \002MODE <#channel> +/-mode <arguments>\002",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :Modes  m                 moderate",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :       r                 regulate",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :       e                 event mode",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       i                 invite only",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       s                 secret (wont show on list)",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :       p                 private",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       n                 must be in chan to send message",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       t                 only ops may change topic",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       b <banmask>       ban management",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       l <limit number>  limit channel number",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       o <nickname>      give/remove ops",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       v <nickname>      give/remove voice",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       k <key>           add/remove channel key",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :\037User\037: \002MODE <yournick> +/-mode\002",
		      me.name, parv[0]);
	  sendto_one (sptr, ":%s 372 %s :       i                 invisible",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       o                 irc operator",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       w                 receive wallops",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       s                 server notices",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       g                 receive globops",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       h                 austnet helper",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       l                 list all channels",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       d                 deaf to channel talk",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       v                 participant in virtual world (using virtual ip's)",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       k                 cant be kicked/deoped(services only)",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       c                 receive client (dis)connect notices(oper only)",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       a                 receive errors and alerts(oper only)",
		      me.name, parv[0]);
	  sendto_one (sptr,
		      ":%s 372 %s :       n                 no auth ident. Replaces the ~ before a username",
		      me.name, parv[0]);

	  return 0;
	}
      else
	{
	  for (i = 0; msgtab[i].cmd; i++)
	    {
	      if ((parv[1][1] && !mycmp (msgtab[i].cmd, parv[1])) ||
		  (!parv[1][1] && msgtab[i].token &&
		   !mycmp (msgtab[i].token, parv[1])))
		{
		  sendto_one (sptr, ":%s 372 %s :\002%-15s %-2s\002 %s",
			      me.name, parv[0], msgtab[i].cmd,
			      (msgtab[i].token ==
			       NULL) ? "" : msgtab[i].token,
			      (msgtab[i].syntax ==
			       NULL) ? "" : msgtab[i].syntax);
		  return 0;
		}
	    }
	  sendto_one (sptr, ":%s 372 %s :%s: Unknown command",
		      me.name, parv[0], parv[1]);
	  return 0;
	}
    }


  for (i = 0; msgtab[i].cmd; i++)
    sendto_one (sptr, ":%s 372 %s :\002%-15s %-2s\002 %s",
		me.name, parv[0], msgtab[i].cmd,
		(msgtab[i].token == NULL) ? "" : msgtab[i].token,
		(msgtab[i].syntax == NULL) ? "" : msgtab[i].syntax);
  return 0;
}
