/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <string.h>
#include "h.h"

static char buf[BUFSIZE];


/*
** m_functions execute protocol messages on this server:
**
**	cptr	is always NON-NULL, pointing to a *LOCAL* client
**		structure (with an open socket connected!). This
**		identifies the physical socket where the message
**		originated (or which caused the m_function to be
**		executed--some m_functions may call others...).
**
**	sptr	is the source of the message, defined by the
**		prefix part of the message if present. If not
**		or prefix not found, then sptr==cptr.
**
**		(!IsServer(cptr)) => (cptr == sptr), because
**		prefixes are taken *only* from servers...
**
**		(IsServer(cptr))
**			(sptr == cptr) => the message didn't
**			have the prefix.
**
**			(sptr != cptr && IsServer(sptr) means
**			the prefix specified servername. (?)
**
**			(sptr != cptr && !IsServer(sptr) means
**			that message originated from a remote
**			user (not local).
**
**		combining
**
**		(!IsServer(sptr)) means that, sptr can safely
**		taken as defining the target structure of the
**		message in this server.
**	*Always* true (if 'parse' and others are working correct):
**
**	1)	sptr->from == cptr  (note: cptr->from == cptr)
**
**	2)	MyConnect(sptr) <=> sptr == cptr (e.g. sptr
**		*cannot* be a local connection, unless it's
**		actually cptr!). [MyConnect(x) should probably
**		be defined as (x == x->from) --msa ]
**
**	parc	number of variable parameter strings (if zero,
**		parv is allowed to be NULL)
**
**	parv	a NULL terminated list of parameter pointers,
**
**			parv[0], sender (prefix string), if not present
**				this points to an empty string.
**			parv[1]...parv[parc-1]
**				pointers to additional parameters
**			parv[parc] == NULL, *always*
**
**		note:	it is guaranteed that parv[0]..parv[parc-1] are all
**			non-NULL pointers.
*/
/*
** m_trace
**	parv[0] = sender prefix
**	parv[1] = servername
*/
/* In thinking about this... Why do users need to trace?  Seems to me
 * it should be for staff only. Modified accordingly --Kwah */

int
m_trace (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
  Reg1 int i;
  Reg2 aClient *acptr;
  aClass *cltmp;
  char *tname;
  int doall, link_s[MAXCONNECTIONS], link_u[MAXCONNECTIONS];
  int cnt = 0, wilds, dow;

  if (check_registered (sptr))
    return 0;
  if (!IsOper (sptr))
    {
      sendto_one (sptr, err_str (ERR_NOPRIVILEGES), me.name, sptr->name);
      return 0;
    }

  if (parc > 2)
    if (hunt_server (cptr, sptr, ":%s TRACE %s :%s", 2, parc, parv))
      return 0;

  if (parc > 1)
    tname = parv[1];
  else
    tname = me.name;

  switch (hunt_server (cptr, sptr, ":%s TRACE :%s", 1, parc, parv))
    {
    case HUNTED_PASS:		/* note: gets here only if parv[1] exists */
      {
	aClient *ac2ptr;

	ac2ptr = next_client (client, tname);
	sendto_one (sptr, rpl_str (RPL_TRACELINK), me.name, parv[0],
		    version, debugmode, tname, ac2ptr->from->name);
	return 0;
      }
    case HUNTED_ISME:
      break;
    default:
      return 0;
    }

  doall = (parv[1] && (parc > 1)) ? !matches (tname, me.name) : TRUE;
  wilds = !parv[1] || index (tname, '*') || index (tname, '?');
  dow = wilds || doall;

  for (i = 0; i < MAXCONNECTIONS; i++)
    link_s[i] = 0, link_u[i] = 0;

  if (doall)
    for (acptr = client; acptr; acptr = acptr->next)
#ifdef	SHOW_INVISIBLE_LUSERS
      if (IsPerson (acptr))
	link_u[acptr->from->fd]++;
#else
      if (IsPerson (acptr) && (!IsInvisible (acptr) || IsOper (sptr)))
	link_u[acptr->from->fd]++;
#endif
      else if (IsServer (acptr))
	link_s[acptr->from->fd]++;

  /* report all direct connections */

  for (i = 0; i <= highest_fd; i++)
    {
      char *name;
      int class;

      if (!(acptr = local[i]))	/* Local Connection? */
	continue;
      if (IsInvisible (acptr) && dow &&
	  !(MyConnect (sptr) && IsOper (sptr)) &&
	  !IsAnOper (acptr) && (acptr != sptr))
	continue;
      if (!doall && wilds && matches (tname, acptr->name))
	continue;
      if (!dow && mycmp (tname, acptr->name))
	continue;
      name = get_client_name (acptr, FALSE);
      class = get_client_class (acptr);

      switch (acptr->status)
	{
	case STAT_CONNECTING:
	  sendto_one (sptr, rpl_str (RPL_TRACECONNECTING), me.name,
		      parv[0], class, name);
	  cnt++;
	  break;
	case STAT_HANDSHAKE:
	  sendto_one (sptr, rpl_str (RPL_TRACEHANDSHAKE), me.name,
		      parv[0], class, name);
	  cnt++;
	  break;
	case STAT_ME:
	  break;
	case STAT_UNKNOWN:
	  sendto_one (sptr, rpl_str (RPL_TRACEUNKNOWN),
		      me.name, parv[0], class, name);
	  cnt++;
	  break;
	case STAT_CLIENT:
	  /* Only opers see users if there is a wildcard
	   * but anyone can see all the opers.
	   */
	  if (IsOper (sptr) &&
	      (MyClient (sptr) || !(dow && IsInvisible (acptr)))
	      || !dow || IsAnOper (acptr))
	    {
	      if (IsAnOper (acptr))
		sendto_one (sptr,
			    rpl_str (RPL_TRACEOPERATOR),
			    me.name,
			    parv[0], class, acptr->name,
			    (IsAnOper (sptr)
			     || !IsVirtual (acptr)) ? acptr->user->
			    host : acptr->user->virthost,
			    now - acptr->lasttime);
	      else
		sendto_one (sptr, rpl_str (RPL_TRACEUSER),
			    me.name, parv[0],
			    class, acptr->name,
			    (IsAnOper (sptr)
			     || !IsVirtual (acptr)) ? acptr->user->
			    host : acptr->user->virthost,
			    now - acptr->lasttime);
	      cnt++;
	    }
	  break;
	case STAT_SERVER:
	  if (acptr->serv->user)
	    sendto_one (sptr, rpl_str (RPL_TRACESERVER),
			me.name, parv[0], class, link_s[i],
			link_u[i], name, acptr->serv->by,
			acptr->serv->user->username,
			acptr->serv->user->host, now - acptr->lasttime);
	  else
	    sendto_one (sptr, rpl_str (RPL_TRACESERVER),
			me.name, parv[0], class, link_s[i],
			link_u[i], name, *(acptr->serv->by) ?
			acptr->serv->by : "*", "*", me.name,
			now - acptr->lasttime);
	  cnt++;
	  break;
	case STAT_SERVICE:
	  sendto_one (sptr, rpl_str (RPL_TRACESERVICE),
		      me.name, parv[0], class, name);
	  cnt++;
	  break;
	case STAT_LOG:
	  sendto_one (sptr, rpl_str (RPL_TRACELOG), me.name,
		      parv[0], LOGFILE, acptr->port);
	  cnt++;
	  break;
	case STAT_PING:
	  sendto_one (sptr, rpl_str (RPL_TRACEPING), me.name,
		      parv[0], name, (acptr->acpt) ?
		      acptr->acpt->name : "<null>");
	  break;
	default:		/* ...we actually shouldn't come here... --msa */
	  sendto_one (sptr, rpl_str (RPL_TRACENEWTYPE), me.name,
		      parv[0], name);
	  cnt++;
	  break;
	}
    }
  /*
   * Add these lines to summarize the above which can get rather long
   * and messy when done remotely - Avalon
   */
  if (!IsAnOper (sptr) || !cnt)
    {
      if (cnt)
	return 0;
      /* let the user have some idea that its at the end of the
       * trace
       */
      sendto_one (sptr, rpl_str (RPL_TRACESERVER),
		  me.name, parv[0], 0, link_s[me.fd],
		  link_u[me.fd], me.name, "*", "*", me.name);
      return 0;
    }
  for (cltmp = FirstClass (); doall && cltmp; cltmp = NextClass (cltmp))
    if (Links (cltmp) > 0)
      sendto_one (sptr, rpl_str (RPL_TRACECLASS), me.name,
		  parv[0], Class (cltmp), Links (cltmp));
  return 0;
}
